/*** DXF-IVF: Konvertierung von ACAD-DXF in IVF ***/
/*   Internet Vektor Format                       */
/*   Version 1.1                                  */
/*   letzte nderungen vom 06.01.98               */


/****** INCLUDES ******/
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

/****** STRUKTUREN *****/
struct typ 					/* TYP Layer, LType, Block zur Umwandlung in EB, LA, PA */
{
	unsigned char name[31]; /* TYPname   im Quellsystem 	     */
	unsigned int  nr,       /* TYPnummer im Zielsystem  	     */
		      la,			/* zugehriger LTYPE (nur bei LAYER) */
		      fa;			/* zugehrige IVF-Farbe				 */
}
LAYER[256],					/* LAYER */
LTYPE[256],                 /* LTYPE */
BLOCK[200];                 /* BLOCK */

struct header				/* Headerelement */
{
	double        x, y;     /* Koordinaten   */
}
H;                          /* Headerelement */

struct element				/* Grafikelement in DXF-Datei	*/
{
	unsigned char eb,		/* Layername   					*/
		      nr;			/* Elementname 					*/
	double        vx,vy,vz, /* Startkoordinaten 			*/
		      nx,ny,nz,		/* Endkoordinaten   			*/
		      r,vw,nw,		/* Radius, Startwinkel, Endwinkel */
		      a;			/* Ausbuchtung 					*/
	unsigned int  bp,		/* Bedeutung Polyline 			*/
		      af;			/* Flag fr Ausbuchtung			*/
}
E;

struct vert					/* Zwischenspeicher fr Vertex-Element */
{
	unsigned int  va,		/* Anfangspunkt der Polyline */
		      af;			/* Flag fr Ausbuchtung      */
	double         a;		/* Ausbuchtung               */
}
V;							/* Zwischenspeicher fr Vertex-Element */

struct punkt				/* Punkt */
{
	unsigned int x, y;		/* Koordinaten */
	unsigned char pa,       /* Punktart    */
		      eb; 			/* Ebene       */
}
PL[65000], 					/* Punktliste  */
P;							/* Einzelpunkt */

struct linie				/* Linie */
{
	unsigned char eb, la;	/* Ebene, Linienart 	 */
	unsigned int  a, e;     /* Anfangs- und Endpunkt */
}
L;							/* Linie */

struct kreis				/* Kreisbogen */
{
	unsigned int  i;		/* Indexnummer der Linie */
	long	      h;		/* Pfeilhhe */
}
K;							/* Kreisbogen */

/****** FUNKTIONEN *****/
void fehler(int err, ...);						/* Gibt Fehlermeldung aus */
unsigned int pindex(struct punkt *PL, struct punkt P, unsigned int p); 	/* Gibt den Punktindex zurck */
unsigned int typnr(char *s, struct typ *T, unsigned int t); 		/* Gibt die Elementnummer zum gesuchten Begriff zurck */
unsigned int indexnr(char *s, struct typ *T, unsigned int t); 		/* Gibt die Indexnummer   zum gesuchten Begriff zurck */
void split(unsigned int z, unsigned char *a);				/* Splittet Zahlen fr IVF[0] */
double strecke(double dx, double dy);					/* Brerechnet die Strecke zwischen zwei Punkten */

/****** MAIN ******/
int main(int argc, char *argv[])
{
	/***** Variablen *****/
	int ivf=0;				/* IVF-Formatnummer
							   0=vorlufiges Format
							   1=IVF1:99 Ebenen,Linienarten,Punktarten
							     40000 Punkte,Linien,Objekte
							     (noch nicht implimentiert)
							   2=IVF2:99 Ebenen,Linienarten,Punktarten
							     100000 Pumkte,Linien,Objekte
							     (noch nicht implimentiert) */
	unsigned int
	max_eb[3]
	={99,99,99},			/* maximale Ebenenanzahl der IVF-Formate */
	max_la[3]
	={99,99,99},        	/* maximale Linienartenanzahl der IVF-Formate */
	max_pa[3]
	={99,99,99},        	/* maximale Punktartenanzahl der IVF-Formate */
	max_punkt[3]
	={6500,40000,65500},	/* maximale Punktanzahl der IVF-Formate */
	max_linie[3]
	={65500,40000,65500},   /* maximale Linienanzahl der IVF-Formate */
	max_koo[3]
	={65500,40000,40000},   /* maximaler Koordinatenbereich der IVF-Formate */
	max_fa[3]
	={99,99,99};			/* maximale Farbenanzahl der IVF-Formate */

	FILE *infile,			/* Zeiger auf Quelldatei */
	     *outfile,			/* Zeiger auf Zieldatei */
	     *insfile,          /* Zeiger auf INSdatei */
	     *linfile,			/* Zeiger aif TMPdatei (LINE) */
	     *arcfile;			/* Zeiger aif TMPdatei (ARC & CIRCLE) */
	char pname[12];			/* Projektname */
	int  ins=1,				/* Installationsdatei 0=Datei nicht vorhanden, 1=Datei vorhanden */
	     ausg_mod=0;		/* Ausgabemodus       0=Normal IVF           , 1=Klartext zum testen */

	char sbuf[256],			/* Schreib-Lesepuffer */
	     buf;				/* Zwischenpuffer */
	unsigned long z,        /* Zeilenindex */
	     filesize;			/* Gre der IVF-Datei */
	long i, j;				/* Laufvariablen */
	unsigned int ibuf;      /* Zwischenvariablen */
	int  nr, 				/* Zwischenvariablen */
	     stz=0,				/* Steuerzeichen in der ungeraden Zeile der DXF-Datei */
	     w_section=0,       /* 0=warte auf SECTION 1=warte auf neue Section */
	     w_table=0,			/* 0=warte auf TABLE   1=warte auf neue Table */
	     header=0,			/* HEADER  1 =$EXTMIN  2 =$EXTMAX */
	     section=0,			/* SECTION 1 =HEADER,  2=TABLES,  3=BLOCKS, 4=ENTITIES, 5=EOF */
	     table=0,			/* TABLE   1 =VPORT ,  2=LTYPE ,  3=LAYER , 4=STYLE   , 5=VIEW     ,  6=DIMSTYLE, 7=UCS   , 8=APPID */
	     element=0,	  		/* Element Gruppe 1 -10, Elemente mit einem Koordinatenpaar
							   1 =POINT ,  2=CIRCLE,  3=TEXT  , 4=SHAPE   , 5=POLYLINE ,  6=VIEWPORT, 7=INSERT, 8=VERTEX, 9=ATTDEF
							   Gruppe 11-20, Elemente mit mehreren Koordinatenpaaren
							   11=LINE  , 12=SOLID , 13=TRACE ,14=3DFACE  ,15=DIMENSION, 16=ARC
							   Gruppe 21-30, Elemente ohne Koordinaten
							   21=SEQEND */
	     layer=0,			/* akt. Layeranzahl  */
	     block=0,			/* akt. Blockanzahl  */
	     ltype=0;          	/* akt. LTypeanzahl  */
	long p=0,				/* akt. Punktanzahl  */
	     l=0,				/* akt. Linienanzahl */
	     k=0,				/* akt. Bogenanzahl  */
	     o=0;				/* akt. Objektanzahl */
	int  vertex=0,			/* akt. VERTEXanzahl */
	     point=0;			/* Punktart fr alle POINT */
	long xmin=99999999,
	     ymin=99999999,
	     xmax=0,
	     ymax=0,			/* Extrema der Eingabedaten */
	     ist_koo;			/* Maximaler Koordinatenbereich in DXF-Datei */
	float f_koo,			/* Faktor, mit denen die reduzierten Koordinaten multipliziert werden */
	ro=3.1415926535/180.;	/* PI */
	unsigned char a[3][3];	/* Zwischenspeicher zum Splitten fr IVF[0] */

	printf("\n");
	/***** Argument bernehmen und prfen *****/
	if(argc<2) fehler(0, pname);	/* Projektname mu angegeben werden */
	for(i=2; i<argc; i++)
	{
		if(!strncmp(argv[i], "-v:", 3))
		{
			ivf=argv[i][3]-48;
			if(ivf>0) fehler(3, argv[i], ivf);
		}
		else if(!strncmp(argv[i], "-t", 2))
			ausg_mod=1;
		else
			fehler(6, argv[i]);
	}

	/***** Vektoren anlegen *****/
/*	if((PL=(struct punkt *)malloc(max_punkt[ivf]*sizeof(P)))==NULL) fehler(5);*/

	/***** Dateien ffnen *****/
	strncpy(pname, argv[1], 9);
	strcat(pname, ".DXF");
	if((infile=fopen(pname, "r"))==NULL) fehler(1, pname);
	strncpy(pname, argv[1], 9);
	strcat(pname, ".INS");
	if((insfile=fopen(pname, "r"))==NULL)
	{
		ins=0;
		if((insfile=fopen(pname, "w"))==NULL) fehler(2, pname);
	}
	else
	{
		strncpy(pname, argv[1], 9);
		strcat(pname, ".IVF");
		if((outfile=fopen(pname, "w"))==NULL) fehler(2, pname);

		if((linfile=tmpfile())==NULL) fehler(7);
		if((arcfile=tmpfile())==NULL) fehler(7);
	}

	/***** INS Lesen wenn vorhanden *****/
	if(ins)
	{
		printf("Lese Installationsdatei %s.INS\n", argv[1]);
		z=1;
		while(!feof(insfile))
		{
			fgets(sbuf, 255, insfile);
			if     (sbuf[0]=='E')
			{
				sscanf(sbuf, "%c %03u %03u %03u %s", &buf, &LAYER[layer].nr, &LAYER[layer].la, &LAYER[layer].fa, &LAYER[layer].name);
				if(LAYER[layer].nr>max_eb[ivf]) fehler(4, z, "Ebenen", ivf);
				if(LAYER[layer].la>max_la[ivf]) fehler(4, z, "Linienarten", ivf);
				if(LAYER[layer].fa>max_fa[ivf]) fehler(4, z, "Farben", ivf);
				layer++;
			}
			else if(sbuf[0]=='L')
			{
				sscanf(sbuf, "%c %03u %s", &buf, &LTYPE[ltype].nr, &LTYPE[ltype].name);
				if(LTYPE[ltype].nr>max_la[ivf]) fehler(4, z, "Linienarten", ivf);
				ltype++;
			}
			else if(sbuf[0]=='B')
			{
				sscanf(sbuf, "%c %03u %s", &buf, &BLOCK[block].nr, &BLOCK[block].name);
				if(BLOCK[block].nr>max_pa[ivf]) fehler(4, z, "Punktarten", ivf);
				block++;
			}
			else if(sbuf[0]=='P')
				sscanf(sbuf, "%c %03u", &buf, &point);
			z++;
		}
	}

	/***** DXF Lesen *****/
	z=1;
	printf("Lese DXF-Datei %s.DXF\n", argv[1]);
	while(!feof(infile))
	{
		fgets(sbuf, 255, infile);
		if(z%2)			/* Steuerzeichenzeile */
		{
			stz=atoi(sbuf);
		}
		else                    /* Datenzeile */
		{
			if(w_section==0)
			{
				if(!strncmp(sbuf, "SECTION", 7)) w_section=1;
			}
			else if(w_section==1)
			{
				if     (!strncmp(sbuf, "HEADER",   6)) {printf("  Lese HEADER\n"  ); section=1; w_section=-1;}
				else if(!strncmp(sbuf, "TABLES",   6)) {printf("  Lese TABLES\n"  ); section=2; w_section=-1;}
				else if(!strncmp(sbuf, "BLOCKS",   6)) {printf("  Lese BLOCKS\n"  ); section=3; w_section=-1;}
				else if(!strncmp(sbuf, "ENTITIES", 8)) {printf("  Lese ENTITIES\n"); section=4; w_section=-1;}
				else if(!strncmp(sbuf, "EOF",      3)) {printf("  Lese EOF\n"     ); section=5; w_section=-1;}
			}
			else
			{
				if     (section==1) /* Header lesen */
				{
					if(stz==0)
					{
						if(!strncmp(sbuf, "ENDSEC" , 6))
						{
							ist_koo=xmax-xmin;
							if((ymax-ymin)>ist_koo) ist_koo=ymax-ymin;      /* maximalen Koordinatenbereich festlegen */
							f_koo=(float)max_koo[ivf]/(float)ist_koo;		/* Faktor zur Transformation in den Koordinatenbereich */
							w_section=1;
						}
					}
					else if(stz==9)
					{
						if(header==1)
						{
							xmin=(long)H.x-1;
							ymin=(long)H.y-1;
						}
						else if(header==2)
						{
							xmax=(long)H.x+1;
							ymax=(long)H.y+1;
						}
						header=0;
						if     (!strncmp(sbuf, "$EXTMIN", 7)) header=1;
						else if(!strncmp(sbuf, "$EXTMAX", 7)) header=2;
					}
					else if(stz==10) sscanf(sbuf,"%lf", &H.x);		/* x-Koordinate */
					else if(stz==20) sscanf(sbuf,"%lf", &H.y);		/* y-Koordinate */
				}
				else if(section==2) /* Tabellen lesen */
				{
					if(w_table==0)
					{
						if(!strncmp(sbuf, "TABLE", 5)) w_table=1;
					}
					else if(w_table==1)
					{
						if     (!strncmp(sbuf, "VPORT",    5)) {table=1; w_table=-1;}
						else if(!strncmp(sbuf, "LTYPE",    5)) {table=2; w_table=-1;}
						else if(!strncmp(sbuf, "LAYER",    5)) {table=3; w_table=-1;}
						else if(!strncmp(sbuf, "STYLE",    5)) {table=4; w_table=-1;}
						else if(!strncmp(sbuf, "VIEW" ,    4)) {table=5; w_table=-1;}
						else if(!strncmp(sbuf, "DIMSTYLE", 8)) {table=6; w_table=-1;}
						else if(!strncmp(sbuf, "UCS",      3)) {table=7; w_table=-1;}
						else if(!strncmp(sbuf, "APPID",    5)) {table=8; w_table=-1;}
						else if(!strncmp(sbuf, "ENDSEC",   6)) {table=0; w_section=1;}
					}
					else
					{
						if     (table==0) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==1) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==2)
						{
							if(stz==0)
							{
								if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;
							}
							else if((stz==2)&&(!ins))
							{
								strncpy(LTYPE[ltype].name, sbuf, 31);
								LTYPE[ltype].nr=ltype;
								ltype++;
							}
						}
						else if(table==3)
						{
							if(stz==0)
							{
								if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;
							}
							else if((stz==2)&&(!ins))
							{
								strncpy(LAYER[layer].name, sbuf, 31);
								LAYER[layer].nr=layer;
							}
							else if((stz==6)&&(!ins))
							{
								LAYER[layer].la=typnr(sbuf, LTYPE, ltype);
								layer++;
							}
						}
						else if(table==4) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==5) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==6) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==7) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
						else if(table==8) {if(stz==0){if(!strncmp(sbuf, "ENDTAB", 6)) w_table=1;}}
					}
				}
				else if(section==3) /* Blocks lesen */
				{
					if(stz==0)
					{
						if(!strncmp(sbuf, "ENDSEC", 6)) w_section=1;
					}
					else if((stz==2)&&(!ins))
					{
						strncpy(BLOCK[block].name, sbuf, 31);
						BLOCK[block].nr=block;
						if(sbuf[0]!='*') block++;	/* Schraffurblcke nicht bernehmen */
					}
				}
				else if(section==4) /* Entities lesen */
				{
					if(stz==0)  /* neues Element */
					{
						/* voriges Element speichern */
						if(element)
						{
							if(element== 1)				/* POINT */
							{
								if(point)
								{
									P.x=(unsigned int)((E.vx-(float)xmin)*f_koo);
									P.y=(unsigned int)((E.vy-(float)ymin)*f_koo);
									if((j=pindex(PL, P, p))==p) p++;
									PL[j].pa=point;
								}
							}
							else if(element== 2)		/* CIRCLE */
							{
								if(((L.la=E.nr)!=0)&&((L.eb=E.eb)!=0))
								{
									P.x=(unsigned int)((E.vx-(float)xmin)*f_koo);
									P.y=(unsigned int)((E.vy-(float)ymin)*f_koo);
									if((L.a=pindex(PL, P, p))==p)
									{
										p++;
										PL[L.a].pa=point;
									}
									L.e = L.a;
									K.h = (unsigned int)(E.r*f_koo);
									K.i=l;
									fwrite(&L, sizeof(L), 1, linfile); l++;
									fwrite(&K, sizeof(K), 1, arcfile); k++;
								}
							}
							else if(element== 3) {}		/* TEXT */
							else if(element== 4) {}     /* SHAPE */
							else if(element== 5)        /* POLYLINE */
							{
								vertex=0;
							}
							else if(element== 6) {}		/* VIEWPORT */
							else if(element== 7)		/* INSERT BlOCK */
							{
								if((E.nr)!=0)
								{
									P.x=(long)((E.vx-(float)xmin)*f_koo);
									P.y=(long)((E.vy-(float)ymin)*f_koo);
									if((j=pindex(PL, P, p))==p) p++;
									PL[j].pa=E.nr;
								}
							}
							else if(element== 8)		/* VERTEX */
							{
								if(((L.la=E.nr)!=0)&&((L.eb=E.eb)!=0))
								{
									if(vertex==0)
									{
										P.x=(unsigned int)((E.vx-(float)xmin)*f_koo);
										P.y=(unsigned int)((E.vy-(float)ymin)*f_koo);
										if((L.a=pindex(PL, P, p))==p)
										{
											p++;
											PL[L.e].pa=point;
										}
										if(E.af){V.a=-E.a; V.af=1;}
										else V.af=0;
										V.va=L.a;
									}
									else
									{
										P.x=(unsigned int)((E.vx-(float)xmin)*f_koo);
										P.y=(unsigned int)((E.vy-(float)ymin)*f_koo);
										if((L.e=pindex(PL, P, p))==p)
										{
											p++;
											PL[L.e].pa=point;
										}
										if(V.af)
										{
											K.h=(strecke((double)PL[L.e].x-(double)PL[L.a].x, (double)PL[L.e].y-(double)PL[L.a].y)*V.a)/2.;
											K.i=l;
											fwrite(&L, sizeof(L), 1, linfile); l++;
											fwrite(&K, sizeof(K), 1, arcfile); k++;
										}
										else
										{
											fwrite(&L, sizeof(L), 1, linfile);
											l++;
										}
										if(E.af){V.a=-E.a; V.af=1;}
										else V.af=0;
										L.a=L.e;
									}
									vertex++;
								}
							}
							else if(element== 9) {}     /* ATTDEF */
							else if(element==11)		/* LINE */
							{
								if(((L.la=E.nr)!=0)&&((L.eb=E.eb)!=0))
								{
									P.x=(unsigned int)((E.vx-(float)xmin)*f_koo);
									P.y=(unsigned int)((E.vy-(float)ymin)*f_koo);
									if((L.a=pindex(PL, P, p))==p)
									{
										p++;
										PL[L.e].pa=point;
									}
									P.x=(unsigned int)((E.nx-(float)xmin)*f_koo);
									P.y=(unsigned int)((E.ny-(float)ymin)*f_koo);
									if((L.e=pindex(PL, P, p))==p)
									{
										p++;
										PL[L.e].pa=point;
									}
									fwrite(&L, sizeof(L), 1, linfile);
									l++;
								}
							}
							else if(element==12) {}     /* SOLID */
							else if(element==13) {}     /* TRACE */
							else if(element==14) {}     /* 3DFACE */
							else if(element==15) {}     /* DIMENSION */
							else if(element==16)		/* ARC */
							{
								if(((L.la=E.nr)!=0)&&((L.eb=E.eb)!=0))
								{
									P.x=(unsigned int)(((E.vx+cos(E.nw)*E.r)-(float)xmin)*f_koo);
									P.y=(unsigned int)(((E.vy+sin(E.nw)*E.r)-(float)ymin)*f_koo);
									if((L.a=pindex(PL, P, p))==p)
									{
										p++;
										PL[L.a].pa=point;
									}
									P.x=(unsigned int)(((E.vx+cos(E.vw)*E.r)-(float)xmin)*f_koo);
									P.y=(unsigned int)(((E.vy+sin(E.vw)*E.r)-(float)ymin)*f_koo);
									if((L.e=pindex(PL, P, p))==p)
									{
										p++;
										PL[L.e].pa=point;
									}
									K.h=(unsigned int)(E.r*(1.-cos((E.nw-E.vw)/2.))*f_koo);
									if((E.nw-E.vw)<0.) K.h=2.*E.r*f_koo-K.h;
									K.i=l;
									fwrite(&L, sizeof(L), 1, linfile); l++;
									fwrite(&K, sizeof(K), 1, arcfile); k++;
								}
							}
							else if(element==21)		/* SEQEND */
							{
								if(E.bp%2)
								{
									L.e=V.va;
									if(V.af)
									{
										K.h=strecke((double)PL[L.e].x-(double)PL[L.a].x, (double)PL[L.e].y-(double)PL[L.a].y)*V.a/2.;
										if(K.h<0.) {K.h=-K.h; ibuf=L.a; L.a=L.e; L.e=ibuf;}
										K.i=l;
										fwrite(&L, sizeof(L), 1, linfile); l++;
										fwrite(&K, sizeof(K), 1, arcfile); k++;
									}
									else
									{
										fwrite(&L, sizeof(L), 1, linfile);
										l++;
									}
								}
								vertex=0;
							}
						}
						/* neuen Elementnamen bernehmen */
						element=0;
						if     (!strncmp(sbuf, "POINT"    , 5)) element= 1;
						else if(!strncmp(sbuf, "CIRCLE"   , 6)) element= 2;
						else if(!strncmp(sbuf, "TEXT"     , 4)) element= 3;
						else if(!strncmp(sbuf, "SHAPE"    , 5)) element= 4;
						else if(!strncmp(sbuf, "POLYLINE" , 8)){element= 5; E.bp=0;}
						else if(!strncmp(sbuf, "VIEWPORT" , 8)) element= 6;
						else if(!strncmp(sbuf, "INSERT"   , 6)) element= 7;
						else if(!strncmp(sbuf, "VERTEX"   , 6)){element= 8; E.af=0;}
						else if(!strncmp(sbuf, "ATTDEF"   , 6)) element= 9;

						else if(!strncmp(sbuf, "LINE"     , 4)) element=11;
						else if(!strncmp(sbuf, "SOLID"    , 5)) element=12;
						else if(!strncmp(sbuf, "TRACE"    , 5)) element=13;
						else if(!strncmp(sbuf, "3DFACE"   , 6)) element=14;
						else if(!strncmp(sbuf, "DIMENSION", 9)) element=15;
						else if(!strncmp(sbuf, "ARC"      , 3)) element=16;

						else if(!strncmp(sbuf, "SEQEND"   , 6)) element=21;

						else if(!strncmp(sbuf, "ENDSEC"   , 6)) w_section=1;
					}
					else if(stz== 2)                     					/* Blockname          */
					{
						E.nr=typnr(sbuf, BLOCK, block);
						if(sbuf[0]=='*') element=0;							/* Schraffurblcke nicht bernehmen */
					}
					else if(stz== 8)										/* LAYER              */
					{
						 E.eb=typnr(sbuf, LAYER, layer);
						 E.nr=LAYER[indexnr(sbuf, LAYER, layer)].la;
					}
					else if(stz== 6) E.nr=typnr(sbuf, LTYPE, ltype);		/* Ltypename          */
					else if(stz==10) sscanf(sbuf,"%lf", &E.vx);				/* x-Koordinate Start */
					else if(stz==20) sscanf(sbuf,"%lf", &E.vy);				/* y-Koordinate Start */
					else if(stz==30) sscanf(sbuf,"%lf", &E.vz);				/* z-Koordinate Start */
					else if(stz==11) sscanf(sbuf,"%lf", &E.nx);				/* x-Koordinate Ende  */
					else if(stz==21) sscanf(sbuf,"%lf", &E.ny);				/* y-Koordinate Ende  */
					else if(stz==31) sscanf(sbuf,"%lf", &E.nz);				/* z-Koordinate Ende  */
					else if(stz==40) sscanf(sbuf,"%lf", &E.r );				/* KreisRadius        */
					else if(stz==42){sscanf(sbuf,"%lf", &E.a ); E.af = 1;}	/* Ausbuchtung        */
					else if(stz==50){sscanf(sbuf,"%lf", &E.vw); E.vw*=ro;}	/* AnfangsWinkel      */
					else if(stz==51){sscanf(sbuf,"%lf", &E.nw); E.nw*=ro;}	/* EndWinkel          */
					else if(stz==70) sscanf(sbuf,"%u ", &E.bp);				/* Bedeutung Polyline */

				}
				else if(section==5) /* Eof lesen */
				{
					if(!strncmp(sbuf, "ENDSEC", 6)) w_section=1;
				}
			}
		}
		z++; /* Zeilenindex++ */
	}

	/***** INS Schreiben wenn nicht vorhanden *****/
	if(!ins)
	{
		printf ("Schreibe Installationsdatei %s.INS\nberprfen Sie die Einstellungen und starten Sie DXF-IVF erneut!", argv[1]);
		fprintf(insfile, "* Installationsdatei Projekt: %s\n* Konvertierung von ACAD-DXF nach IVF(%i)\n*\n", argv[1], ivf);
		fprintf(insfile, "* Layer werden als Ebenen bersetzt, Maximal: %i, E 000 wird nicht bertragen\n*\n", max_eb[ivf]);
		for(i=0; i<layer; i++)
			fprintf(insfile, "E %03u 001 %03u %s", LAYER[i].nr, LAYER[i].nr, LAYER[i].name);
		fprintf(insfile, "*\n* LTYPES werden als Linienarten bersetzt, Maximal %i, L 000 wird nicht bertragen\n*\n", max_la[ivf]);
		for(i=0; i<ltype; i++)
			fprintf(insfile, "L %03u %s", 1, LTYPE[i].name);
		fprintf(insfile, "*\n* alle POINTS werden als Punktart ### bersetzt, P 000 wird nicht bertragen\n*\n");
		fprintf(insfile, "P 000\n");
		fprintf(insfile, "*\n* BLOCKS werden als Punktarten bersetzt Maximal %i, B 000 wird nicht bertragen\n*\n", max_pa[ivf]);
		for(i=0; i<block; i++)
			fprintf(insfile, "B %03u %s", 0, BLOCK[i].name);
		fprintf(insfile, "*\n");
	}
	/***** IVF Schreiben *****/
	else
	{
		printf("Schreibe IVF-Datei %s.IVF ", argv[1]);
		rewind(linfile); rewind(arcfile);

		if(ausg_mod)	/* im Textmodus */
		{
			printf("im Textmodus\n");
			if(ivf==0)			/* IVF[0] */
			{
				fprintf(outfile, "* Punkte\n");
				for(i=0; i<p; i++)
					fprintf(outfile, "%5u %02u %02u %10.3f %10.3f\n", i, PL[i].eb, PL[i].pa, (float)(PL[i].y)/f_koo+(float)ymin, (float)(PL[i].x)/f_koo+(float)xmin);
				fprintf(outfile, "* Linien\n");
				for(i=0; i<l; i++)
				{
					fread(&L, sizeof(L), 1, linfile);
					fprintf(outfile, "%5u %2u %2u %5u %5u\n", i, L.eb, L.la, L.a, L.e);
				}
				fprintf(outfile, "* Kreisbgen\n");
				for(i=0; i<k; i++)
				{
					fread(&K, sizeof(K), 1, arcfile);
					fprintf(outfile, "%5u %5u %10.3f\n", i, K.i, (float)(K.h)/f_koo);
				}
			}
			else if(ivf==1)		/* IVF[1] */
			{

			}
			else if(ivf==2)     /* IVF[2] */
			{

			}
		}
		else			/* im Normal-Modus */
		{
			printf("\n");
			if(ivf==0)			/* IVF[0] */
			{
				fprintf(outfile, "%c\n%c\n", 26, 61);     	/* Header schreiben */
				fprintf(outfile, "%c\n%s\n", 20, argv[1]);
				split(max_koo[ivf], a[0]);
				fprintf(outfile, "%c\n%c%c%c%c%c%c", 21, a[0][0], a[0][1], a[0][2], a[0][0], a[0][1], a[0][2]);
				split(floor(f_koo), a[0]);
				split((unsigned int)((f_koo-floor(f_koo))*max_koo[ivf]), a[1]);
				fprintf(outfile, "%c%c%c%c%c%c\n", a[0][0], a[0][1], a[0][2], a[1][0], a[1][1], a[1][2]);
				fprintf(outfile, "%c\n", 25);
				filesize=34;
				for(i=0; i<layer; i++)                      /* Ebenen schreiben */
				{
					fprintf(outfile, "%c%c%s\n", LAYER[i].nr+50, LAYER[i].fa+50, LAYER[i].name);
					filesize+=4+strlen(LAYER[i].name);
				}
				fprintf(outfile, "%c\n", 22);				/* Punkte schreiben */

				for(i=0; i<p; i++)
				{
					split(             PL[i].x, a[0]);
					split(max_koo[ivf]-PL[i].y, a[1]);
					fprintf(outfile, "%c%c%c%c%c%c%c", PL[i].pa+50, a[0][0], a[0][1], a[0][2], a[1][0], a[1][1], a[1][2]);
					if(!((i+1)%30)) fprintf(outfile, "\n");
				}
				printf("  Punkte: %u\n", p);
				filesize+=p*7+p/30*2;
				if(l>0)
				{
					i=1;
					fprintf(outfile, "\n%c\n", 23);			/* Linien schreiben */
					for(i=0; i<l; i++)
					{
						fread(&L, sizeof(L), 1, linfile);
						split(L.a, a[0]);
						split(L.e, a[1]);
						fprintf(outfile, "%c%c%c%c%c%c%c", L.eb+50, a[0][0], a[0][1], a[0][2], a[1][0], a[1][1], a[1][2]);
						if(!((i+1)%30)) fprintf(outfile, "\n");
					}
				}
				printf("  Linien: %u\n", l-k);
				filesize+=l*7+l/30*2;
				if(k>0)
				{
					i=1;
					fprintf(outfile, "\n%c\n", 24);
					for(i=0; i<k; i++)
					{
						fread(&K, sizeof(K), 1, arcfile);
						split(K.i, a[0]);
						if(K.h<0)
						{
							a[1][0]=51;
							K.h=(unsigned int)(-K.h);
						}
						else
						{
							a[1][0]=50;
						}
						split(K.h, a[2]);
						fprintf(outfile, "%c%c%c%c%c%c%c",  a[0][0], a[0][1], a[0][2], a[1][0], a[2][0], a[2][1], a[2][2]);
						if(!((i+1)%30)) fprintf(outfile, "\n");
					}
				}
				fprintf(outfile, "\n%c\n", 49);
				printf("  Bgen : %u\n", k);
				filesize+=k*6+k/35*2;
				printf("Dateigre: %lukb", filesize/1024);
			}
			else if(ivf==1)			/* IVF[1] */
			{

			}
			else if(ivf==2)         /* IVF[2] */
			{

			}
		}
	}
	/***** ENDE *****/
	return 0;
}

/****** FUNKTIONEN ******/
void fehler(int err, ...)
{
	va_list ap;
	va_start(ap, err);
	switch(err)
	{
		case  0: printf("DXF-IVF --- Umsetzung von AutoCad-DXF nach IVF (Internet Vektor Format)\n");
			 printf("Aufruf: DXF-IVF <projekt>\n");
			 printf("          <projekt>          Projektname\n");
			 printf("                             <projekt>.dxf wird erwartet\n");
			 printf("Optionen: -v:<VerNum>        IVF-Versionsnummer\n");
			 printf("                             0=vorlufiges Format\n");
			 printf("          -t                 Ausgabe erfolgt im Textmodus\n"); break;
		case  1: printf("Kann Datei %s nicht zum Lesen ffnen!\n", va_arg(ap, int)); break;
		case  2: printf("Kann Datei %s nicht zum Schreiben ffnen!\n", va_arg(ap, int)); break;
		case  3: printf("Falscher Parameter %s, ",va_arg(ap, int));
			 printf("IVF-Version %i nicht implimentiert!\n", va_arg(ap, int)); break;
		case  4: printf("Fehler in Zeile %i, ",va_arg(ap, long));
			 printf("Maximale %sanzahl ", va_arg(ap, int));
			 printf("fr IVF-Format %u berschritten!", va_arg(ap, int)); break;
		case  5: printf("Fehler bei der Speicherzuweisung\n"); break;
		case  6: printf("Unbekannte Option %s\n", va_arg(ap, int)); break;
		case  7: printf("Kann temporre Dateien nicht ffnen!\n"); break;

		default: printf("E:%i Undef. Fehler!\n",err);
	}
	va_end(ap);
	exit(err);
}
unsigned int pindex(struct punkt *PL, struct punkt P, unsigned int p)
{
	long i;
	for(i=0; i<p; i++)
	{
		if(PL[i].x==P.x)
			if(PL[i].y==P.y) return(i);
	}
	PL[p]=P;
	return p;
}
unsigned int typnr(char *s, struct typ *T, unsigned int t)
{
	int i, l;
	l=strlen(s)-1;
	for(i=0; i<t; i++)
		if(!strncmp(s, T[i].name, l)) return(T[i].nr);
	return (0);
}
unsigned int indexnr(char *s, struct typ *T, unsigned int t)
{
	int i, l;
	l=strlen(s)-1;
	for(i=0; i<t; i++)
		if(!strncmp(s, T[i].name, l)) return(i);
	return(-1);
}
void split(unsigned int z, unsigned char *a)
{
	a[0]=z/10000;
	a[1]=(z-(a[0]*10000))/100;
	a[2]=z-a[0]*10000-a[1]*100;
	a[0]+=50; a[1]+=50; a[2]+=50;
	return;
}
double strecke(double dx, double dy)
{
	return(sqrt(dx*dx+dy*dy));
}